// CthulhuDoc.cpp : implementation of the CCthulhuDoc class
//

#include "stdafx.h"
#include "MainFrm.h"
#include "Cthulhu.h"
#include "HTMLExport.h"
#include "CthulhuDoc.h"
#include "PlayerNameDlg.h"
#include "NameEtcDlg.h"
#include "AddWeaponDlg.h"
#include "NewSkillDlg.h"
#include "CharacteristicsDlg.h"
#include "EditSkillsDlg.h"
#include "NewCharacter_Wizard.h"
#include "CheckInvestigator.h"
#include "InputNumberDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//picture loading stuff
extern char* ReadBitmap( CString strFilename, UINT& uWidth, UINT& uHeight, HPALETTE& hPalette );


/////////////////////////////////////////////////////////////////////////////
// CCthulhuDoc

IMPLEMENT_DYNCREATE(CCthulhuDoc, CDocument)

BEGIN_MESSAGE_MAP(CCthulhuDoc, CDocument)
	//{{AFX_MSG_MAP(CCthulhuDoc)
	ON_COMMAND(ID_FILE_EXPORT_HTML, OnFileExportHtml)
	ON_COMMAND(ID_EDIT_CHARACTERISTICS_EDIT, OnEditCharacteristics)
	ON_COMMAND(ID_EDIT_CHARACTERISTICS_RANDOMISE, OnRandomiseCharacteristics)
	ON_COMMAND(ID_EDIT_GENERALINFORMATION_EDIT, OnEditGeneralinformation)
	ON_COMMAND(ID_EDIT_PLAYERINFORMATION_EDIT, OnEditPlayerinformation)
	ON_COMMAND(ID_EDIT_PORTRAIT_BROWSE, OnPortraitBrowse)
	ON_COMMAND(ID_EDIT_PORTRAIT_CLEAR, OnPortraitClear)
	ON_COMMAND(ID_EDIT_SKILLS_EDIT, OnEditSkills)
	ON_COMMAND(ID_EDIT_WEAPONS_ADD, OnAddWeapon)
	ON_COMMAND(ID_EDIT_WEAPONS_CLEAR, OnClearWeapons)
	ON_UPDATE_COMMAND_UI(ID_EDIT_WEAPONS_CLEAR, OnUpdateClearWeapons)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PORTRAIT_CLEAR, OnUpdatePortraitClear)
	ON_COMMAND(ID_EDIT_SKILLS_ADD, OnAddSkill)
	ON_COMMAND(ID_TOOLS_CHECKINVESTIGATOR, OnToolsCheckInvestigator)
	ON_COMMAND(ID_TOOLS_INCOME, OnToolsIncome)
	ON_COMMAND(ID_FILE_SEND_MAIL, OnFileSendMail)
	ON_UPDATE_COMMAND_UI(ID_FILE_SEND_MAIL, OnUpdateFileSendMail)
	ON_COMMAND(ID_EDIT_HITPOINTS_EDIT, OnEditHitpointsEdit)
	ON_COMMAND(ID_EDIT_HITPOINTS_RESET, OnEditHitpointsReset)
	ON_COMMAND(ID_EDIT_MAGIC_EDIT, OnEditMagicEdit)
	ON_COMMAND(ID_EDIT_MAGIC_RESET, OnEditMagicReset)
	ON_COMMAND(ID_EDIT_SANITY_EDIT, OnEditSanityEdit)
	ON_COMMAND(ID_EDIT_SANITY_RESET, OnEditSanityReset)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCthulhuDoc construction/destruction

CCthulhuDoc::CCthulhuDoc()
{
    m_fNeedRedrawRecalculation = TRUE;

    m_pBitmap = NULL;
    m_hPalPortrait = NULL;
    m_uWidth = m_uHeight = 0;

    m_hEmfEra = NULL;

	m_Investigator.m_era = ERA_1920; //default era

	m_pUndoList = m_pRedoList = NULL;
	m_nUndo = m_nRedo = 0;
}

CCthulhuDoc::~CCthulhuDoc()
{
    if( m_pBitmap ) 
    {
        GlobalUnlock( m_pBitmap  );
        GlobalFree( m_pBitmap  );
        m_pBitmap = NULL;
    }

    if( m_hPalPortrait ) DeleteObject( m_hPalPortrait );

    if( m_hEmfEra ) DeleteEnhMetaFile(m_hEmfEra); 

    //delete undo & redo lists
	LPUNDOREDOLIST pTemp;
	while( m_pRedoList )
	{
		pTemp = m_pRedoList;
		m_pRedoList = m_pRedoList->next;
		delete pTemp;
	}
	while( m_pUndoList )
	{
		pTemp = m_pUndoList;
		m_pUndoList = m_pUndoList->next;
		delete pTemp;
	}
}

BOOL CCthulhuDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

    //delete undo & redo lists, if any (this may not be needed as it's an MDI app now)
	LPUNDOREDOLIST pTemp;
	while( m_pRedoList )
	{
		pTemp = m_pRedoList;
		m_pRedoList = m_pRedoList->next;
		delete pTemp;
	}
	while( m_pUndoList )
	{
		pTemp = m_pUndoList;
		m_pUndoList = m_pUndoList->next;
		delete pTemp;
	}

    //get user's log on name
	char szUserName[1024];
	DWORD dwLength = 1023;
	GetUserName( szUserName, &dwLength );
	m_Investigator.m_strPlayerName = szUserName;

	//basic attributes
	m_Investigator.m_strName = "";
	m_Investigator.m_strOccupation = "";
	m_Investigator.m_strSex = "";
	m_Investigator.m_strColleges = "";
	m_Investigator.m_strBirthplace = "";
	m_Investigator.m_strMentalDisorders	= "";
    m_Investigator.m_strPortrait = "";

	//characteristics: -1 is never displayed
	m_Investigator.m_nAPP = -1;
	m_Investigator.m_nSTR = -1;
	m_Investigator.m_nCON = -1;
	m_Investigator.m_nDEX = -1;
	m_Investigator.m_nPOW = -1;
	m_Investigator.m_nSIZ = -1;
	m_Investigator.m_nINT = -1;
	m_Investigator.m_nEDU = -1;

    //age: -1 is never displayed
	m_Investigator.m_nAge = -1;

    //advanced characteristics: tables only circle in-range values...
    m_Investigator.m_nSAN = -999;
    m_Investigator.m_nHitPoints = -999;
    m_Investigator.m_nMagic = -999;
    
    //skills - add all basic according to era
    m_Investigator.DeleteSkills();
    m_Investigator.SetOccupationSkills( NULL /* no occupation... */ );

    //update all "current" skill values
    m_fNeedRedrawRecalculation = TRUE;

    //load the portrait & era pictures
    LoadEraPicture();
    LoadPortraitFile();

	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CCthulhuDoc serialization

void CCthulhuDoc::Serialize(CArchive& ar)
{
    //store/read basic stats
    m_Investigator.Serialize( ar );

    //store/read player name
    if( !ar.IsStoring() )
    {
        //load the pictures
        LoadPortraitFile();
        LoadEraPicture();

        //redraw everything
        m_fNeedRedrawRecalculation = TRUE;
	    UpdateAllViews( NULL );
    }

}

/////////////////////////////////////////////////////////////////////////////
// CCthulhuDoc diagnostics

#ifdef _DEBUG
void CCthulhuDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CCthulhuDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CCthulhuDoc commands

BOOL CCthulhuDoc::ShowMiniWizard()
{
	//run the character generation wizard
	CNewCharacterWizard newCharacter;
    newCharacter.m_Investigator.m_era = m_Investigator.m_era;
    newCharacter.m_fShowFullWizard = FALSE;

	if( newCharacter.DoModal() != IDCANCEL )
	{
        //they didn't cancel - retrive all data from the investigator
        m_Investigator = newCharacter.m_Investigator;
        LoadPortraitFile();
        LoadEraPicture();

        //change the document title to represent this investigator
		SetTitle( m_Investigator.m_strName );
        SetModifiedFlag();

	    //display changes
        m_Investigator.SortSkillList();
        m_fNeedRedrawRecalculation = TRUE;
	    UpdateAllViews( NULL );

        return TRUE;
	}
    else
        return FALSE;
}

BOOL CCthulhuDoc::ShowFullWizard()
{
	//run the character generation wizard
	CNewCharacterWizard newCharacter;
    newCharacter.m_Investigator.m_era = m_Investigator.m_era;
    newCharacter.m_fShowFullWizard = TRUE;

	if( newCharacter.DoModal() != IDCANCEL )
	{
        //they didn't cancel - retrive all data from the investigator
        m_Investigator = newCharacter.m_Investigator;
        LoadPortraitFile();
        LoadEraPicture();

        //change the document title to represent this investigator
		SetTitle( m_Investigator.m_strName );
        SetModifiedFlag();

	    //display changes
        m_Investigator.SortSkillList();
        m_fNeedRedrawRecalculation = TRUE;
	    UpdateAllViews( NULL );

        return TRUE;
	}
    else
        return FALSE;
}


void CCthulhuDoc::LoadPortraitFile()
{
    if( m_pBitmap ) 
    {
        GlobalUnlock( m_pBitmap  );
        GlobalFree( m_pBitmap  );
        m_pBitmap = NULL;
    }
    if( m_hPalPortrait ) DeleteObject( m_hPalPortrait );

    if( m_Investigator.m_strPortrait.GetLength() > 0 )
    {
        //don't bother to check the file extension as there is only one type (bmp) supported...
        m_pBitmap = ReadBitmap( m_Investigator.m_strPortrait, m_uWidth, m_uHeight, m_hPalPortrait );
    }
}

void CCthulhuDoc::LoadEraPicture()
{
    //delete existing era picture
    if( m_hEmfEra ) DeleteEnhMetaFile(m_hEmfEra); 

    //load new era picture
    CString strEraPic = GetDefaultDirectory();

    switch( m_Investigator.m_era )
    {
        case ERA_1890: strEraPic += "1890.emf"; break;
        default:
        case ERA_1920: strEraPic += "1920.emf"; break;
        case ERA_1990: strEraPic += "1990.emf"; break;
    }

    //load the meta file
    m_hEmfEra = GetEnhMetaFile( strEraPic );
}

void CCthulhuDoc::OnFileExportHtml() 
{
    ExportToHTML( &m_Investigator );	
}

void CCthulhuDoc::OnEditCharacteristics() 
{
	CCharacteristicsDlg Characteristics;

    Characteristics.m_nAPP = m_Investigator.m_nAPP;
    Characteristics.m_nCON = m_Investigator.m_nCON;
    Characteristics.m_nDEX = m_Investigator.m_nDEX;
    Characteristics.m_nPOW = m_Investigator.m_nPOW;
    Characteristics.m_nSTR = m_Investigator.m_nSTR;
    Characteristics.m_nSIZ = m_Investigator.m_nSIZ;
    Characteristics.m_nINT = m_Investigator.m_nINT;
    Characteristics.m_nEDU = m_Investigator.m_nEDU;

    if( Characteristics.DoModal() == IDOK )
    {
        //store an undo
        StoreUndo( "Modify Characteristics" );

        //set investigator's new values
        m_Investigator.m_nAPP = Characteristics.m_nAPP;
        m_Investigator.m_nCON = Characteristics.m_nCON;
        m_Investigator.m_nDEX = Characteristics.m_nDEX;
        m_Investigator.m_nPOW = Characteristics.m_nPOW;
        m_Investigator.m_nSTR = Characteristics.m_nSTR;
        m_Investigator.m_nSIZ = Characteristics.m_nSIZ;
        m_Investigator.m_nINT = Characteristics.m_nINT;
        m_Investigator.m_nEDU = Characteristics.m_nEDU;

        if( m_Investigator.m_nSAN == -999 ) m_Investigator.m_nSAN = m_Investigator.m_nPOW*5;
        if( m_Investigator.m_nHitPoints == -999 ) m_Investigator.m_nHitPoints = (int)( 0.5 + (((double)m_Investigator.m_nCON + (double)m_Investigator.m_nSIZ ) / 2.0 ) );
        if( m_Investigator.m_nMagic == -999 ) m_Investigator.m_nMagic = m_Investigator.m_nPOW;

        //the investigator has been modified
        SetModifiedFlag();

	    //display changes
        m_fNeedRedrawRecalculation = TRUE;
	    UpdateAllViews( NULL );
    }
}

void CCthulhuDoc::OnRandomiseCharacteristics() 
{
    //store an undo
    StoreUndo( "Generate Random Characteristics" );

    //generate random values
	m_Investigator.m_nAPP = GetValueInRange( D6_MIN, D6_MAX, 3 );
	m_Investigator.m_nCON = GetValueInRange( D6_MIN, D6_MAX, 3 );
	m_Investigator.m_nDEX = GetValueInRange( D6_MIN, D6_MAX, 3 );
	m_Investigator.m_nPOW = GetValueInRange( D6_MIN, D6_MAX, 3 );
	m_Investigator.m_nSTR = GetValueInRange( D6_MIN, D6_MAX, 3 );

	m_Investigator.m_nSIZ = 6 + GetValueInRange( D6_MIN, D6_MAX, 2 );
	m_Investigator.m_nINT = 6 + GetValueInRange( D6_MIN, D6_MAX, 2 );

	m_Investigator.m_nEDU = 3 + GetValueInRange( D6_MIN, D6_MAX, 3 );

    m_Investigator.m_nAge = m_Investigator.m_nEDU + 6;

    m_Investigator.m_nSAN = m_Investigator.m_nPOW*5;
    m_Investigator.m_nMagic = m_Investigator.m_nPOW;
    m_Investigator.m_nHitPoints = (int)( 0.5 + (((double)m_Investigator.m_nCON + (double)m_Investigator.m_nSIZ ) / 2.0 ) );

    //the investigator has been modified
    SetModifiedFlag();

	//display changes
    m_fNeedRedrawRecalculation = TRUE;
	UpdateAllViews( NULL );
}

void CCthulhuDoc::OnEditGeneralinformation() 
{
    CNameEtcDlg	NameEtc;

    NameEtc.m_nAge = m_Investigator.m_nAge;
    NameEtc.m_strBirthplace = m_Investigator.m_strBirthplace;
    NameEtc.m_strColleges = m_Investigator.m_strColleges;
    NameEtc.m_strMentalDisorders = m_Investigator.m_strMentalDisorders;
    NameEtc.m_strName = m_Investigator.m_strName;
    NameEtc.m_strOccupation = m_Investigator.m_strOccupation;
    NameEtc.m_strSex = m_Investigator.m_strSex;

    if( NameEtc.DoModal() == IDOK )
    {
        //store an undo
        StoreUndo( "Modify General Information" );

        //extract new info from dialog
        m_Investigator.m_nAge = NameEtc.m_nAge;
        m_Investigator.m_strBirthplace = NameEtc.m_strBirthplace;
        m_Investigator.m_strColleges = NameEtc.m_strColleges;
        m_Investigator.m_strMentalDisorders = NameEtc.m_strMentalDisorders;
        m_Investigator.m_strName = NameEtc.m_strName;
        m_Investigator.m_strOccupation = NameEtc.m_strOccupation;
        m_Investigator.m_strSex = NameEtc.m_strSex;

        //the investigator has been modified
        SetModifiedFlag();

	    //display changes
        m_fNeedRedrawRecalculation = TRUE;
	    UpdateAllViews( NULL );
    }
}

void CCthulhuDoc::OnEditPlayerinformation() 
{
	CPlayerNameDlg PlayerName;
    PlayerName.m_strPlayerName = m_Investigator.m_strPlayerName;
    if( PlayerName.DoModal() == IDOK )
    {
        m_Investigator.m_strPlayerName = PlayerName.m_strPlayerName;

        //the investigator has been modified
        SetModifiedFlag();

	    //display changes
        m_fNeedRedrawRecalculation = TRUE;
	    UpdateAllViews( NULL );
    }
}

void CCthulhuDoc::OnPortraitBrowse() 
{
	//browse for a new portrait
    OPENFILENAME ofn;
    char szFilename[MAX_PATH+1];
    LPCTSTR lpstrFileMasks = "Bitmap Files\0*.bmp\0All Files (*.*)\0*.*\0\0";

    //get the current file name as default
    strcpy( szFilename, LPCSTR(m_Investigator.m_strPortrait) );
    
    //browse for a new one
    ofn.lStructSize = sizeof(OPENFILENAME);
    ofn.hwndOwner = AfxGetMainWnd()->m_hWnd;
    ofn.hInstance = NULL;
    ofn.lpstrFilter = lpstrFileMasks;
    ofn.lpstrCustomFilter = NULL;
    ofn.nFilterIndex = 1;
    ofn.lpstrFile = szFilename;
    ofn.nMaxFile = MAX_PATH;
    ofn.lpstrFileTitle = NULL;
    ofn.nMaxFileTitle = 0;
    ofn.lpstrInitialDir = NULL;
    ofn.lpstrTitle = "Select Portrait";
    ofn.Flags = OFN_EXPLORER|OFN_HIDEREADONLY|OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST;
    ofn.lpstrDefExt = "BMP";
    if( GetOpenFileName( &ofn ) )
    {
        //store an undo
        StoreUndo( "Change Portrait" );

        //set the portrait
        m_Investigator.m_strPortrait = ofn.lpstrFile;
        LoadPortraitFile();

        //the investigator has been modified
        SetModifiedFlag();

	    //display changes
        m_fNeedRedrawRecalculation = TRUE;
	    UpdateAllViews( NULL );
    }

	
}

void CCthulhuDoc::OnPortraitClear() 
{
    //store an undo
    StoreUndo( "Clear Portrait" );

    //clear the portrait
    m_Investigator.m_strPortrait = "";
    LoadPortraitFile();

    //the investigator has been modified
    SetModifiedFlag();

	//display changes
    m_fNeedRedrawRecalculation = TRUE;
	UpdateAllViews( NULL );
}

void CCthulhuDoc::OnAddWeapon() 
{
	CAddWeaponDlg AddWeapon;
    AddWeapon.m_era = m_Investigator.m_era;
    AddWeapon.m_pInvestigator = &m_Investigator;
    if( AddWeapon.DoModal() == IDOK )
    {
        //store an undo
        StoreUndo( "Add Weapon" );

        //add the skill
        CInvestigator::InvestigatorSkillList* pNew = m_Investigator.AddSkill( AddWeapon.m_pSkillToAdd, FALSE );
        if( pNew != NULL )
        {
            pNew->nCurrent = pNew->nMinimumValue = m_Investigator.GetSkillNumber( pNew->pSkill->m_strBase );
            m_Investigator.SortSkillList();

            //the investigator has been modified
            SetModifiedFlag();

	        //display changes
            m_fNeedRedrawRecalculation = TRUE;
	        UpdateAllViews( NULL );
        }
    }
}

void CCthulhuDoc::OnAddSkill() 
{
    CNewSkillDlg NewSkill;

    if( NewSkill.DoModal() == IDOK )
    {
        //store an undo
        StoreUndo( "Add Skill" );

        CString strBase;
        strBase.Format( "%d", NewSkill.m_nBase );
        CSkill* pSkill = new CSkill( NewSkill.m_strName, strBase, TRUE, TRUE, TRUE );

        CInvestigator::InvestigatorSkillList* pNew = m_Investigator.AddSkill( pSkill, FALSE );
        if( pNew == NULL )
        {
            delete pSkill;
            return;
        }
        else
        {
            pNew->nCurrent = pNew->nMinimumValue = m_Investigator.GetSkillNumber( strBase );
            m_Investigator.SortSkillList();
        }

        //the investigator has been modified
        SetModifiedFlag();

	    //display changes
        m_fNeedRedrawRecalculation = TRUE;
	    UpdateAllViews( NULL );
    }
}


void CCthulhuDoc::OnClearWeapons() 
{
    //store an undo
    StoreUndo( "Clear Weapons" );

    //clear the weapons
    m_Investigator.ClearWeapons();
	
    //the investigator has been modified
    SetModifiedFlag();

	//display changes
    m_fNeedRedrawRecalculation = TRUE;
	UpdateAllViews( NULL );
}

void CCthulhuDoc::OnUpdateClearWeapons(CCmdUI* pCmdUI) 
{
    //count investigator's weapons
    CInvestigator::InvestigatorSkillList* pList = m_Investigator.GetList();
    int nWeapons = 0;
    while( pList )
    {
        //if it is a firearm or melee weapon, remove it
        if( (pList->pSkill->GetType() == MELEE || pList->pSkill->GetType() == FIREARM) && pList->pSkill->m_strName[0] != '*' )
            nWeapons++;

        //advance list
        pList = pList->next;
    }

    //toggle enable/disable
    pCmdUI->Enable( nWeapons > 0 );
}

void CCthulhuDoc::OnUpdatePortraitClear(CCmdUI* pCmdUI) 
{
    pCmdUI->Enable( m_Investigator.m_strPortrait.GetLength() > 0 );	
}

void CCthulhuDoc::OnEditSkills() 
{
    //store an undo
    StoreUndo( "Modify Skills" );

    CEditSkillsDlg EditSkills;
    EditSkills.m_pInvestigator = &m_Investigator;
    EditSkills.DoModal();

    //the investigator has been modified
    SetModifiedFlag();

	//display changes
    m_Investigator.SortSkillList();
    m_fNeedRedrawRecalculation = TRUE;
	UpdateAllViews( NULL );

}


void CCthulhuDoc::StoreUndo( CString strName )
{
    //delete redo list
	LPUNDOREDOLIST pTemp;
	while( m_pRedoList )
	{
		pTemp = m_pRedoList;
		m_pRedoList = m_pRedoList->next;
		delete pTemp;
	}
    m_nRedo = 0;

	//store the current state
	LPUNDOREDOLIST pUndo = new UNDOREDOLIST;
    pUndo->strActionDesc = strName;
    pUndo->Investigator = m_Investigator;
	pUndo->next = m_pUndoList;
	m_pUndoList = pUndo;

	//trim undo list
	m_nUndo++;
	if( m_nUndo > MAX_UNDO_SIZE )
	{
		//remove last item
		pUndo = m_pUndoList;
		while( pUndo->next->next != NULL ) pUndo = pUndo->next;
		delete pUndo->next;
		pUndo->next = NULL;
		m_nUndo--;
	}

    //update menu
    UpdateUndoRedoMenu();
}

void CCthulhuDoc::Undo()
{
	if( m_nUndo > 0 )
	{
		//duplicate investigator
        CInvestigator InvestigatorCurrent = m_Investigator;

        //do the undo
        m_Investigator = m_pUndoList->Investigator;

		//move undo to redo
		LPUNDOREDOLIST pUndoRedo = m_pUndoList;
		m_pUndoList = m_pUndoList->next;

		pUndoRedo->next = m_pRedoList;
		m_pRedoList = pUndoRedo;
		m_nRedo++;
		m_nUndo--;

		//remove current undo & redo and set new
        pUndoRedo->Investigator = InvestigatorCurrent;
		
		if( m_nRedo > MAX_UNDO_SIZE )
		{
			//remove last item
			pUndoRedo = m_pRedoList;
			while( pUndoRedo->next->next != NULL ) pUndoRedo = pUndoRedo->next;
			delete pUndoRedo->next;
			pUndoRedo->next = NULL;
			m_nRedo--;
		}

        //the investigator has been modified
        LoadPortraitFile();
        SetModifiedFlag();

	    //display changes
        m_fNeedRedrawRecalculation = TRUE;
	    UpdateAllViews( NULL );
        UpdateUndoRedoMenu();
    } 
}

void CCthulhuDoc::Redo() 
{
	if( m_nRedo > 0 )
	{
		//duplicate investigator
        CInvestigator InvestigatorCurrent = m_Investigator;

		//do the redo
        m_Investigator = m_pRedoList->Investigator;

		//move undo to redo
		LPUNDOREDOLIST pUndoRedo = m_pRedoList;
		m_pRedoList = m_pRedoList->next;

		pUndoRedo->next = m_pUndoList;
		m_pUndoList = pUndoRedo;
		m_nUndo++;
		m_nRedo--;

		//remove current undo & redo and set new
        pUndoRedo->Investigator = InvestigatorCurrent;

        //the investigator has been modified
        LoadPortraitFile();
        SetModifiedFlag();

	    //display changes
        m_fNeedRedrawRecalculation = TRUE;
	    UpdateAllViews( NULL );
        UpdateUndoRedoMenu();
	}
}

void CCthulhuDoc::UpdateUndoRedoMenu()
{
	//change the menu descriptions
	CMenu* pMenu = AfxGetMainWnd()->GetMenu();
	if( m_pUndoList )
	{
		CString str;
        str.Format( "&Undo %s\tCtrl+Z", m_pUndoList->strActionDesc );
		pMenu->ModifyMenu( ID_EDIT_UNDO, MF_BYCOMMAND|MF_STRING, ID_EDIT_UNDO, LPCSTR(str) );
	}
	else pMenu->ModifyMenu( ID_EDIT_UNDO, MF_BYCOMMAND|MF_STRING, ID_EDIT_UNDO, "&Undo\tCtrl+Z" );

	if( m_pRedoList )
	{
		CString str;
        str.Format( "&Redo %s\tCtrl+Y", m_pRedoList->strActionDesc );
		pMenu->ModifyMenu( ID_EDIT_REDO, MF_BYCOMMAND|MF_STRING, ID_EDIT_REDO, LPCSTR(str) );
	}
	else pMenu->ModifyMenu( ID_EDIT_REDO, MF_BYCOMMAND|MF_STRING, ID_EDIT_REDO, "&Redo\tCtrl+Y" );
}


BOOL CCthulhuDoc::IsUndo() 
{
    return ( m_pUndoList != NULL ); 
}

BOOL CCthulhuDoc::IsRedo()
{ 
    return ( m_pRedoList != NULL ); 
}

void CCthulhuDoc::OnToolsCheckInvestigator() 
{
    //let the dialog handle everything
    CCheckInvestigator CheckInvestigator;
    CheckInvestigator.m_pInvestigator = &m_Investigator;
    CheckInvestigator.DoModal();
}


void CCthulhuDoc::UpdateCalculatedBases()
{
    CInvestigator::InvestigatorSkillList* pSkills = m_Investigator.GetList();
    while( pSkills )
    {
        //calculate minimum value & update current if it's lower
        if( pSkills->pSkill->IsCalculatedBase() )
        {
            pSkills->nMinimumValue = m_Investigator.GetSkillNumber( pSkills->pSkill->m_strBase );
            if( pSkills->nCurrent < pSkills->nMinimumValue ) pSkills->nCurrent = pSkills->nMinimumValue;
        }

        pSkills = pSkills->next;
    }
}

void CCthulhuDoc::OnToolsIncome() 
{
    int nIncomeBase = 0, nIncomeStep = 0;
    switch( m_Investigator.m_era )
    {
        case ERA_1890: nIncomeBase = 5000; nIncomeStep = 5000; break;
        case ERA_1920: nIncomeBase = 4500; nIncomeStep = 1000; break;
        case ERA_1990: nIncomeBase = 15000; nIncomeStep = 10000; break;
        default:
            ASSERT(FALSE);
    }

    int nValue = GetValueInRange( D10_MIN, D10_MAX, 1 ) - 1;
    int nIncome = nIncomeBase + (nValue * nIncomeStep );

    CString strMessage;
    strMessage.Format( "You investigator has an income of $%d", nIncome );

    AfxMessageBox( strMessage, MB_ICONINFORMATION );
}


void CCthulhuDoc::OnEditHitpointsEdit() 
{
    CInputNumberDlg InputNumber;
    InputNumber.m_strPrompt = "Please enter the investigator's hit points";
    if( m_Investigator.m_nHitPoints == -999 )
    {
        if( m_Investigator.m_nCON > 0 && m_Investigator.m_nSIZ > 0 )
            InputNumber.m_nNumber = (int)( 0.5 + (((double)m_Investigator.m_nCON + (double)m_Investigator.m_nSIZ ) / 2.0 ) );
        else
            InputNumber.m_nNumber = 0;
    }
    else
        InputNumber.m_nNumber = m_Investigator.m_nHitPoints;

    if( InputNumber.DoModal() == IDOK )
    {
        //store an undo
        StoreUndo( "Modify Hit Points" );

        //modify value
        m_Investigator.m_nHitPoints = InputNumber.m_nNumber;

        //the investigator has been modified
        SetModifiedFlag();

	    //display changes
        m_Investigator.SortSkillList();
        m_fNeedRedrawRecalculation = TRUE;
	    UpdateAllViews( NULL );
    }
}

void CCthulhuDoc::OnEditMagicEdit() 
{
    CInputNumberDlg InputNumber;
    InputNumber.m_strPrompt = "Please enter the investigator's magic points";
    if( m_Investigator.m_nMagic == -999 )
    {
        if( m_Investigator.m_nPOW > 0 )
            InputNumber.m_nNumber = m_Investigator.m_nPOW;
        else
            InputNumber.m_nNumber = 0;
    }
    else
        InputNumber.m_nNumber = m_Investigator.m_nMagic;
    if( InputNumber.DoModal() == IDOK )
    {
        //store an undo
        StoreUndo( "Modify Magic Points" );

        //modify value
        m_Investigator.m_nMagic = InputNumber.m_nNumber;

        //the investigator has been modified
        SetModifiedFlag();

	    //display changes
        m_Investigator.SortSkillList();
        m_fNeedRedrawRecalculation = TRUE;
	    UpdateAllViews( NULL );
    }
}

void CCthulhuDoc::OnEditSanityEdit() 
{
    CInputNumberDlg InputNumber;
    InputNumber.m_strPrompt = "Please enter the investigator's SAN";
    if( m_Investigator.m_nSAN == -999 )
    {
        if( m_Investigator.m_nPOW > 0 )
        {
            InputNumber.m_nNumber = m_Investigator.m_nPOW*5;
            CInvestigator::InvestigatorSkillList* pCthulhuMythos = m_Investigator.FindSkill( "Cthulhu Mythos" );
            if( pCthulhuMythos )
            {
                int n99_CthulhuMythos = (99-pCthulhuMythos->nCurrent);
                if( InputNumber.m_nNumber > n99_CthulhuMythos ) InputNumber.m_nNumber = n99_CthulhuMythos;
            }
        }
        else
            InputNumber.m_nNumber = 0;
    }
    else
        InputNumber.m_nNumber = m_Investigator.m_nSAN;

    if( InputNumber.DoModal() == IDOK )
    {
        //store an undo
        StoreUndo( "Modify SAN" );

        //modify value
        m_Investigator.m_nSAN = InputNumber.m_nNumber;

        //the investigator has been modified
        SetModifiedFlag();

	    //display changes
        m_Investigator.SortSkillList();
        m_fNeedRedrawRecalculation = TRUE;
	    UpdateAllViews( NULL );
    }
}

void CCthulhuDoc::OnEditHitpointsReset() 
{
    if( AfxMessageBox( "Are you sure you want to reset the investigator's hit points?", MB_ICONQUESTION|MB_YESNO ) == IDYES )
    {
        //store an undo
        StoreUndo( "Reset Hit Points" );

        //modify value
        m_Investigator.m_nHitPoints = (int)( 0.5 + (((double)m_Investigator.m_nCON + (double)m_Investigator.m_nSIZ ) / 2.0 ) );

        //the investigator has been modified
        SetModifiedFlag();

	    //display changes
        m_Investigator.SortSkillList();
        m_fNeedRedrawRecalculation = TRUE;
	    UpdateAllViews( NULL );
    }
}

void CCthulhuDoc::OnEditMagicReset() 
{
    if( AfxMessageBox( "Are you sure you want to reset the investigator's magic points?", MB_ICONQUESTION|MB_YESNO ) == IDYES )
    {
        //store an undo
        StoreUndo( "Reset Magic Points" );

        //modify value
        m_Investigator.m_nMagic = m_Investigator.m_nPOW;

        //the investigator has been modified
        SetModifiedFlag();

	    //display changes
        m_Investigator.SortSkillList();
        m_fNeedRedrawRecalculation = TRUE;
	    UpdateAllViews( NULL );
    }
}

void CCthulhuDoc::OnEditSanityReset() 
{
    if( AfxMessageBox( "Are you sure you want to reset the investigator's sanity points?", MB_ICONQUESTION|MB_YESNO ) == IDYES )
    {
        //store an undo
        StoreUndo( "Reset Sanity" );

        //modify value
        m_Investigator.m_nSAN = m_Investigator.m_nPOW*5;

        //find Cthulhu Mythos skill & shrink SAN if needed
        CInvestigator::InvestigatorSkillList* pCthulhuMythos = m_Investigator.FindSkill( "Cthulhu Mythos" );
        if( pCthulhuMythos )
        {
            int n99_CthulhuMythos = (99-pCthulhuMythos->nCurrent);
            if( m_Investigator.m_nSAN > n99_CthulhuMythos ) m_Investigator.m_nSAN = n99_CthulhuMythos;
        }

        //the investigator has been modified
        SetModifiedFlag();

	    //display changes
        m_Investigator.SortSkillList();
        m_fNeedRedrawRecalculation = TRUE;
	    UpdateAllViews( NULL );
    }
}
