// CthulhuView.h : interface of the CCthulhuView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_CTHULHUVIEW_H__B534ECCD_A26B_11D1_89C0_00001C192944__INCLUDED_)
#define AFX_CTHULHUVIEW_H__B534ECCD_A26B_11D1_89C0_00001C192944__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "UserTextEdit.h"

class CCthulhuView : public CScrollView
{
protected: // create from serialization only
	CCthulhuView();
	DECLARE_DYNCREATE(CCthulhuView)

// Attributes
public:
	CCthulhuDoc* GetDocument();

    enum SheetHitTest { HT_NONE, HT_ERA, HT_NAME, HT_STAT, HT_SIDE, HT_SAN, HT_MAGIC, HT_HIT, HT_SKILL, HT_PIC, HT_HAND, HT_FIRE };

    enum SheetDisplayType { DT_COC50, DT_COC50SMALL, DT_DELTAGREEN };
    SheetDisplayType m_DisplayType;
    
    CUserTextEdit m_EditControl;

	CRect RectFromBox( SheetHitTest ht );
    SheetHitTest BoxFromPoint( LPPOINT pt );

    void LoadSettings();
    void UpdateSettings();

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CCthulhuView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	//}}AFX_VIRTUAL

// Implementation
public:
	void SheetToView( CRect& rect );
	void ViewToSheet( CRect& rect );
	void SheetToView( CPoint& point );
	void ViewToSheet( CPoint& point );
	virtual ~CCthulhuView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CCthulhuView)
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnEditShowText();
	afx_msg void OnUpdateEditShowText(CCmdUI* pCmdUI);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnEditCopy();
	afx_msg void OnEditCut();
	afx_msg void OnEditPaste();
	afx_msg void OnEditUndo();
	afx_msg void OnEditRedo();
	afx_msg void OnUpdateEditUndo(CCmdUI* pCmdUI);
	afx_msg void OnUpdateEditRedo(CCmdUI* pCmdUI);
	afx_msg void OnEditNextField();
	afx_msg void OnEditPrevField();
    afx_msg void OnSheetStyleChange( UINT uID );
    afx_msg void OnUpdateSheetStyle(CCmdUI* pCmdUI);
	afx_msg void OnViewWhatSheetType();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
private:
	void DrawDGCheckBox( CDC* pDC, int x, int y );
	void DrawDGStatBox( CDC* pDC, int x, int y, CString strCaption, int* nValue, BOOL fEdit = FALSE );
    void DrawField(CDC * pDC, CString strName, int x1, int x2, int y, CString strText, int nTextOffset, UINT uExtraFormat );
    void DrawField(CDC * pDC, CString strName, int x1, int x2, int y, int nText, int nTextOffset, UINT uExtraFormat );
    void DrawUserField(CDC * pDC, int x1, int x2, int y, int nText, UINT uExtraFormat, BOOL fDrawLine = TRUE, int nAllowLeft = 0 );
	void DrawUserField( CDC* pDC, int x1, int x2, int y, CString strText, UINT uExtraFormat, BOOL fDrawLine = TRUE, int nAllowLeft = 0 );

    void DrawV5CharSheet( CDC* pDC );
    void DrawV5CharSheetSmall( CDC* pDC );
    void DrawDeltaGreenCharSheet( CDC* pDC  );

    void SetSheetSize();
    void GetSheetSize( CPoint& pt );

    void LoadMetafiles();
	void DrawCaptionBox( CDC* pDC, CString strCaption, CRect rcCaption );
    void DrawPortrait( CDC* pDC );

    int m_iRotateType;
    int m_nTopMargin;
    CRect m_rcEra, m_rcName, m_rcStat, m_rcSide, m_rcSan, m_rcMagic, m_rcHit, m_rcSkill, m_rcPic, m_rcHand, m_rcFire;

    //user-customisable bits
	CString m_strUserTextFace;
    CString m_strTitleFace;
    CString m_strDeltaGreenFace;
    COLORREF m_crUserText;
    BOOL m_fShowText;

    //off-screen drawing stuff
    CDC m_DCOffscreen;
    CBitmap m_BmOffscreen;

    //metafiles
    HENHMETAFILE m_hEmfSide;
    HENHMETAFILE m_hEmfDeltaGreen;


    struct FieldList {
        CString* pstr;
        int* pn;
        int nConstraintValue;
        CRect rc;
        CString strDescription;

        struct FieldList* next;
        struct FieldList* prev;

        int iField;
    };
    FieldList* m_pFieldList;
    int m_iCurrent;
    int m_nFieldCount;

    void AddField( int* pn, int x1, int x2, int y, CString strDescription, int nMinValue = 0 );
    void AddField( CString* pstr, int x1, int x2, int y, CString strDescription, int nMaxLength = 0 );
    void AddFieldListItem( FieldList* pItem );
    FieldList* FindField( LPPOINT lppt );
    FieldList* FindField( int i );
    void DeleteFieldList();
    void BeginEditing( FieldList* pField );
};

#ifndef _DEBUG  // debug version in CthulhuView.cpp
inline CCthulhuDoc* CCthulhuView::GetDocument()
   { return (CCthulhuDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_CTHULHUVIEW_H__B534ECCD_A26B_11D1_89C0_00001C192944__INCLUDED_)
