// NewCharacter_Pages.cpp : implementation file
//

#include "stdafx.h"
#include <shlobj.h>
#include "resource.h"
#include "Cthulhu.h"
#include "CharacterAge.h"
#include "NewCharacter_Pages.h"
#include "NewCharacter_Wizard.h"
#include "SpecialityNameDlg.h"
#include "PickSkill.h"
#include "skill.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(CNewCharacter_Outline, CPropertyPage)
IMPLEMENT_DYNCREATE(CNewCharacter_Occupation, CPropertyPage)
IMPLEMENT_DYNCREATE(CNewCharacter_PersonalInterest, CPropertyPage)
IMPLEMENT_DYNCREATE(CNewCharacter_Weapons, CPropertyPage)
IMPLEMENT_DYNCREATE(CNewCharacter_Skills, CPropertyPage)
IMPLEMENT_DYNCREATE(CNewCharacter_NameEtc, CPropertyPage)
IMPLEMENT_DYNCREATE(CNewCharacter_Finished, CPropertyPage)

/////////////////////////////////////////////////////////////////////////////
// Global Helper Functions
void DrawSkillList( CWnd* pWnd, int nIDCtl, LPDRAWITEMSTRUCT lpDrawItemStruct )
{
    //draw selection background colour
    if( lpDrawItemStruct->itemState & ODS_SELECTED )
    {
        FillRect( lpDrawItemStruct->hDC, &lpDrawItemStruct->rcItem, (HBRUSH)(COLOR_HIGHLIGHT+1) );
        SetBkColor( lpDrawItemStruct->hDC, GetSysColor(COLOR_HIGHLIGHT) );
        SetTextColor( lpDrawItemStruct->hDC, GetSysColor(COLOR_HIGHLIGHTTEXT) );
    }
    else
    {
        FillRect( lpDrawItemStruct->hDC, &lpDrawItemStruct->rcItem, (HBRUSH)(COLOR_WINDOW+1) );
        SetBkColor( lpDrawItemStruct->hDC, GetSysColor(COLOR_WINDOW) );
        SetTextColor( lpDrawItemStruct->hDC, GetSysColor(COLOR_WINDOWTEXT) );
    }

    //draw the text
    CInvestigator::InvestigatorSkillList* pSkill = (CInvestigator::InvestigatorSkillList*)lpDrawItemStruct->itemData;
	if( pSkill )
	{
		//set the font
		SetTextAlign( lpDrawItemStruct->hDC, TA_LEFT|VTA_BASELINE );
        HFONT hFontOld = NULL;
		if( pSkill->fOccupational ) //draw occupational skills differently
		{
			//get the current font
			CFont* pFont = pWnd->GetFont();

			//get the LOGFONT and toggle bold
			LOGFONT lf;
			pFont->GetLogFont( &lf );
			lf.lfWeight = ( lf.lfWeight == 0 || lf.lfWeight == FW_NORMAL || lf.lfWeight == FW_REGULAR ) ? FW_BOLD : FW_NORMAL;

			//create a new font based on it
			CFont NewFont;
			NewFont.CreateFontIndirect( &lf );
			hFontOld = (HFONT)SelectObject( lpDrawItemStruct->hDC, HFONT(NewFont) );
		}

        CString strText;
		if( pSkill->pSkill->GetType() == SPECIALISESKILLITEM )
		{
            CSpecialiseSkillSkill* pSpecial = (CSpecialiseSkillSkill*)pSkill->pSkill;

            strText = pSpecial->m_pSkill->m_strName;
			strText += ": ";
			strText += pSpecial->m_strName;

            if( pSpecial->m_strName.GetLength() == 0 )
            {
				SIZE sz;
				GetTextExtentPoint32( lpDrawItemStruct->hDC, LPCSTR(strText), strText.GetLength(), &sz );
				
				RECT rc;
				rc.top = lpDrawItemStruct->rcItem.top + 5;
				rc.left = lpDrawItemStruct->rcItem.left + 8 + sz.cx;
				rc.right = lpDrawItemStruct->rcItem.right - 48;
				rc.bottom = lpDrawItemStruct->rcItem.bottom - 5;
				DrawFocusRect( lpDrawItemStruct->hDC, &rc );
            }
		}
        else
		    strText = pSkill->pSkill->m_strName;

        if( pSkill->pSkill->GetType() == ANYSKILL )
		{
			RECT rc;
			rc.top = lpDrawItemStruct->rcItem.top + 5;
			rc.left = lpDrawItemStruct->rcItem.left + 4;
			rc.right = lpDrawItemStruct->rcItem.right - 48;
			rc.bottom = lpDrawItemStruct->rcItem.bottom - 5;
			DrawFocusRect( lpDrawItemStruct->hDC, &rc );
			rc.top = lpDrawItemStruct->rcItem.top + 5;
			rc.left = lpDrawItemStruct->rcItem.right - 40;
			rc.right = lpDrawItemStruct->rcItem.right - 4;
			rc.bottom = lpDrawItemStruct->rcItem.bottom - 5;
			DrawFocusRect( lpDrawItemStruct->hDC, &rc );
			strText = ""; 
		}

        if( strText.GetLength() > 0 && strText[0] == '*' )
        {
            const char * pszName = LPCSTR(strText);
            while( *pszName == '*' ) pszName++;
            CString strNewText = pszName;

		    //draw the text, without a leading '*'
		    TextOut( lpDrawItemStruct->hDC, 2+lpDrawItemStruct->rcItem.left, lpDrawItemStruct->rcItem.top + ((lpDrawItemStruct->rcItem.bottom - lpDrawItemStruct->rcItem.top) / 2 ) + 3, strNewText, strNewText.GetLength() );
        }
        else
		    //draw the text
		    TextOut( lpDrawItemStruct->hDC, 2+lpDrawItemStruct->rcItem.left, lpDrawItemStruct->rcItem.top + ((lpDrawItemStruct->rcItem.bottom - lpDrawItemStruct->rcItem.top) / 2 ) + 3, LPCSTR(strText), strText.GetLength() );

        //put the old font back again
        if( hFontOld ) SelectObject( lpDrawItemStruct->hDC, hFontOld );



		//draw the skill chance
		if( pSkill->pSkill->GetType() != ANYSKILL )
		{
			strText.Format( "%d%%", pSkill->nCurrent );
			SetTextAlign( lpDrawItemStruct->hDC, TA_LEFT|VTA_BASELINE );
			TextOut( lpDrawItemStruct->hDC, lpDrawItemStruct->rcItem.right - 36, lpDrawItemStruct->rcItem.top + ((lpDrawItemStruct->rcItem.bottom - lpDrawItemStruct->rcItem.top) / 2 ) + 3, LPCSTR(strText), strText.GetLength() );
		}
	}

    //draw the focus rectangle
    if( lpDrawItemStruct->itemState & ODS_FOCUS ) DrawFocusRect( lpDrawItemStruct->hDC, &lpDrawItemStruct->rcItem );
}


/////////////////////////////////////////////////////////////////////////////
// CNewCharacter_Outline property page

CNewCharacter_Outline::CNewCharacter_Outline() : CPropertyPage(CNewCharacter_Outline::IDD)
{
	//{{AFX_DATA_INIT(CNewCharacter_Outline)
	m_nAPP = 0;
	m_nCON = 0;
	m_nDEX = 0;
	m_nEDU = 0;
	m_nINT = 0;
	m_nPOW = 0;
	m_nSIZ = 0;
	m_nSTR = 0;
	m_strDB = _T("");
	m_strHP = _T("");
	m_strIdea = _T("");
	m_strKnow = _T("");
	m_strLuck = _T("");
	m_strSAN = _T("");
	m_nAge = 0;
	m_strPersonalInterest = _T("");
	m_strSkillPoints = _T("");
	//}}AFX_DATA_INIT

	m_nSkillPoints = 0;
	m_nPersonalInterestPoints = 0;

	m_fRotted = FALSE;
	m_fHadMoreSchooling = FALSE;

	//initialise the values
	GenerateNewStats();

	m_fFinishedThis = FALSE;
}

CNewCharacter_Outline::~CNewCharacter_Outline()
{
}

void CNewCharacter_Outline::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CNewCharacter_Outline)
	DDX_Control(pDX, IDC_AGE, m_wndAge);
	DDX_Control(pDX, IDC_AGESPIN, m_wndAgeSpin);
	DDX_Control(pDX, IDC_EDITSTR, m_wndEditStr);
	DDX_Control(pDX, IDC_EDITSIZ, m_wndEditSiz);
	DDX_Control(pDX, IDC_EDITPOW, m_wndEditPow);
	DDX_Control(pDX, IDC_EDITINT, m_wndEditInt);
	DDX_Control(pDX, IDC_EDITEDU, m_wndEditEdu);
	DDX_Control(pDX, IDC_EDITDEX, m_wndEditDex);
	DDX_Control(pDX, IDC_EDITCON, m_wndEditCon);
	DDX_Control(pDX, IDC_EDITAPP, m_wndEditApp);
	DDX_Text(pDX, IDC_EDITAPP, m_nAPP);
	DDX_Text(pDX, IDC_EDITCON, m_nCON);
	DDX_Text(pDX, IDC_EDITDEX, m_nDEX);
	DDX_Text(pDX, IDC_EDITEDU, m_nEDU);
	DDX_Text(pDX, IDC_EDITINT, m_nINT);
	DDX_Text(pDX, IDC_EDITPOW, m_nPOW);
	DDX_Text(pDX, IDC_EDITSIZ, m_nSIZ);
	DDX_Text(pDX, IDC_EDITSTR, m_nSTR);
	DDX_Text(pDX, IDC_DB, m_strDB);
	DDX_Text(pDX, IDC_HP, m_strHP);
	DDX_Text(pDX, IDC_IDEA, m_strIdea);
	DDX_Text(pDX, IDC_KNOW, m_strKnow);
	DDX_Text(pDX, IDC_LUCK, m_strLuck);
	DDX_Text(pDX, IDC_SAN, m_strSAN);
	DDX_Text(pDX, IDC_AGE, m_nAge);
	DDX_Text(pDX, IDC_PERSONALINTEREST, m_strPersonalInterest);
	DDX_Text(pDX, IDC_SKILLPOINTS, m_strSkillPoints);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CNewCharacter_Outline, CPropertyPage)
	//{{AFX_MSG_MAP(CNewCharacter_Outline)
	ON_BN_CLICKED(IDC_REROLL, OnReroll)
	ON_EN_KILLFOCUS(IDC_AGE, OnKillfocusAge)
	ON_NOTIFY(UDN_DELTAPOS, IDC_AGESPIN, OnDeltaPosAgeSpin)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CNewCharacter_Outline::OnReroll() 
{
	m_wndAgeSpin.EnableWindow( TRUE );
	m_wndAge.EnableWindow( TRUE );
	m_fFinishedThis = FALSE;

	GenerateNewStats();

	//pop the new values onto the dialog
	UpdateData(FALSE);

	m_wndAgeSpin.SetRange( m_nBaseEDU+6, MAX_AGE );
}

void CNewCharacter_Outline::UpdateOtherValues()
{
	//(fraction is rounded up)
	int nHP = (int)( 0.5 + (((double)m_nCON + (double)m_nSIZ ) / 2.0 ) );
	m_strHP.Format( "%d", nHP );

	//update damage bonus
	m_strDB = GetDBString( m_nSTR, m_nSIZ );

	//calculate starting edu and minimum age
	m_nEDU = m_nBaseEDU + ( m_fSchoolingWorked ? 1 : 0 );
	int nMinAge = m_nBaseEDU+6;
	int nAge = m_nAge;

	//set the minimum age for the spin control - don't change the value in the box, though
	if( nAge < nMinAge ) 
	{
		nAge = nMinAge;
		MessageBeep(0xFFFFFFFF);
	}
	else
		if( nAge > MAX_AGE )
		{
			nAge = MAX_AGE;
			MessageBeep(0xFFFFFFFF);
		}

	//increase EDU
	int nDifference = ( nAge - nMinAge ) / 10;
	m_nEDU += nDifference;

	//calculate the number of experience and personal interest points
	m_nSkillPoints = m_nBaseEDU * 20;
    m_nPersonalInterestPoints = (m_nINT * 10) + (m_fSchoolingWorked ? 10 : 0 );

	//display the values
	m_strPersonalInterest.Format( "%d", m_nPersonalInterestPoints + (m_fSchoolingWorked ? 10 : 0 ) );
	m_strSkillPoints.Format( "%d", m_nSkillPoints ); 

	//update calculated fields
	m_strLuck.Format( "%d", m_nPOW * 5 );
	m_strKnow.Format( "%d", m_nEDU * 5 );
	m_strIdea.Format( "%d", m_nINT * 5 );
	m_strSAN.Format( "%d", m_nPOW * 5 );

}


void CNewCharacter_Outline::GenerateNewStats()
{
	//generate a new set of values
	m_nAPP = GetValueInRange( D6_MIN, D6_MAX, 3 );
	m_nCON = GetValueInRange( D6_MIN, D6_MAX, 3 );
	m_nDEX = GetValueInRange( D6_MIN, D6_MAX, 3 );
	m_nPOW = GetValueInRange( D6_MIN, D6_MAX, 3 );
	m_nSTR = GetValueInRange( D6_MIN, D6_MAX, 3 );

	m_nSIZ = 6 + GetValueInRange( D6_MIN, D6_MAX, 2 );
	m_nINT = 6 + GetValueInRange( D6_MIN, D6_MAX, 2 );

	m_nEDU = m_nBaseEDU = 3 + GetValueInRange( D6_MIN, D6_MAX, 3 );

	m_fRotted = FALSE;
	m_fSchoolingWorked = FALSE;
	m_fHadMoreSchooling = FALSE;

	//set the age to the minimum age allowed
	m_nAge = m_nBaseEDU+6;

	//update calculated values
	UpdateOtherValues();
}

BOOL CNewCharacter_Outline::OnSetActive() 
{
	((CPropertySheet*)GetParent())->SetWizardButtons( PSWIZB_NEXT );
	m_wndAgeSpin.SetRange( m_nBaseEDU+6, MAX_AGE );

	return CPropertyPage::OnSetActive();
}

LRESULT CNewCharacter_Outline::OnWizardNext() 
{

	UpdateData(TRUE);

	if( m_fFinishedThis == FALSE )
	{
		if( m_fHadMoreSchooling == FALSE )
		{
			if( AfxMessageBox( "Do you wish to spend one extra year in schooling?", MB_YESNO|MB_ICONQUESTION ) == IDYES )
			{
				m_fHadMoreSchooling = TRUE;

				if( GetValueInRange( D100_MIN, D100_MAX, 1 ) <= (m_nINT*5) )
				{
					AfxMessageBox( "Your investigator learned some more personal interest skills from staying on a year", MB_ICONINFORMATION );
					m_fSchoolingWorked = TRUE;

				} else AfxMessageBox( "Your investigator did not learn anything extra from staying on a year", MB_ICONINFORMATION );
			}
		}

		UpdateOtherValues();

		if( m_fRotted == FALSE && m_nAge + (m_fHadMoreSchooling ? 1 : 0 ) > 40 )
		{
			//FIXME: ok on 50, 60, 70 etc?
			int nDifference = ( m_nAge + (m_fHadMoreSchooling ? 1 : 0 )) / 10;
			nDifference -= 3;

			int nAPPRot = 0;
			int nDEXRot = 0;
			int nCONRot = 0;
			int nSTRRot = 0;
			while( nDifference )
			{
				CCharacterAge CharAgeDlg;
				if( CharAgeDlg.DoModal() == IDOK )
				{
					switch( CharAgeDlg.m_WhatToRot )
					{
						case CCharacterAge::ROT_APP: nAPPRot++; break;
						case CCharacterAge::ROT_DEX: nDEXRot++; break;
						case CCharacterAge::ROT_CON: nCONRot++; break;
						case CCharacterAge::ROT_STR: nSTRRot++; break;
						default: ASSERT(FALSE);
					}
				}
				else
				{
					UpdateOtherValues();
					m_fHadMoreSchooling = FALSE;
					m_fSchoolingWorked = FALSE;
					UpdateData(FALSE);
					return -1;
				}

				nDifference--;
			}

			m_nAPP -= nAPPRot;
			m_nDEX -= nDEXRot;
			m_nCON -= nCONRot;
			m_nSTR -= nSTRRot;

			m_fRotted = TRUE;
		}

		m_wndAgeSpin.EnableWindow( FALSE );
		m_wndAge.EnableWindow( FALSE );
		m_fFinishedThis = TRUE;

		if( m_fHadMoreSchooling ) m_nAge++;

		UpdateOtherValues();
		UpdateData(FALSE);
	}

	//write the settings back to the parent
	CNewCharacterWizard* pWizard = (CNewCharacterWizard*)GetParent();
	
	pWizard->m_nSkillPoints = m_nSkillPoints;
	pWizard->m_nPersonalInterestPoints = m_nPersonalInterestPoints;
	pWizard->m_Investigator.m_nAge = m_nAge;
	pWizard->m_Investigator.m_nAPP = m_nAPP;
	pWizard->m_Investigator.m_nCON = m_nCON;
	pWizard->m_Investigator.m_nDEX = m_nDEX;
	pWizard->m_Investigator.m_nEDU = m_nEDU;
	pWizard->m_Investigator.m_nINT = m_nINT;
	pWizard->m_Investigator.m_nPOW = m_nPOW;
	pWizard->m_Investigator.m_nSIZ = m_nSIZ;
	pWizard->m_Investigator.m_nSTR = m_nSTR;

    pWizard->m_Investigator.m_nHitPoints = (int)( 0.5 + (((double)m_nCON + (double)m_nSIZ ) / 2.0 ) );
    pWizard->m_Investigator.m_nSAN = m_nPOW*5;
    pWizard->m_Investigator.m_nMagic = m_nPOW;

	//go to next page
	return CPropertyPage::OnWizardNext();
}

void CNewCharacter_Outline::OnKillfocusAge() 
{
	UpdateData(TRUE);

	//prevent age from going below allowed minimum
	if( m_nAge < m_nBaseEDU+6 )
		m_nAge = m_nBaseEDU+6;
	else
		if( m_nAge > MAX_AGE ) m_nAge = MAX_AGE;

	UpdateOtherValues();
	UpdateData(FALSE);
	
}

void CNewCharacter_Outline::OnDeltaPosAgeSpin(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	// TODO: Add your control notification handler code here

	UpdateData();

	//set the range for the control
	m_wndAgeSpin.SetRange( m_nBaseEDU+6, MAX_AGE );

	UpdateOtherValues();
	UpdateData(FALSE);
	
	*pResult = 0;
}













/////////////////////////////////////////////////////////////////////////////
// CNewCharacter_Occupation property page

CNewCharacter_Occupation::CNewCharacter_Occupation() : CPropertyPage(CNewCharacter_Occupation::IDD)
{
	//count the occupations
	int nOccupations = g_Occupations.GetCount();


	//{{AFX_DATA_INIT(CNewCharacter_Occupation)
	m_iOccupation = nOccupations ? (rand() % nOccupations - 1) + 1 : -1;
	//}}AFX_DATA_INIT
}

CNewCharacter_Occupation::~CNewCharacter_Occupation()
{
}

void CNewCharacter_Occupation::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CNewCharacter_Occupation)
	DDX_Control(pDX, IDC_OCCUPATION, m_wndOccupation);
	DDX_CBIndex(pDX, IDC_OCCUPATION, m_iOccupation);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CNewCharacter_Occupation, CPropertyPage)
	//{{AFX_MSG_MAP(CNewCharacter_Occupation)
		// NOTE: the ClassWizard will add message map macros here
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


BOOL CNewCharacter_Occupation::OnSetActive() 
{
	((CPropertySheet*)GetParent())->SetWizardButtons( PSWIZB_BACK|PSWIZB_NEXT );
	
	//clear the list box and add the custom occupation item
	int i;
	m_wndOccupation.ResetContent();

	//add all of the default occupations
	CNewCharacterWizard* pWizard = ((CNewCharacterWizard*)GetParent());
	ERA era = pWizard->m_Investigator.m_era;

    COccupationList::OccupationList* pOccupations = g_Occupations.GetList();
	while( pOccupations )
	{
        if( pOccupations->pOccupation->IsEra( era ) )
        {
            const char* pszName = LPCSTR(pOccupations->pOccupation->m_strName);
            i = m_wndOccupation.AddString( *pszName == '*' ? ++pszName : pszName );
		    m_wndOccupation.SetItemData( i, (DWORD)pOccupations->pOccupation );
        }
		pOccupations = pOccupations->next;
	}

	m_wndOccupation.SetCurSel( m_iOccupation );

	return CPropertyPage::OnSetActive();
}


LRESULT CNewCharacter_Occupation::OnWizardNext() 
{
	UpdateData();

	//write the settings back to the parent
	CNewCharacterWizard* pWizard = ((CNewCharacterWizard*)GetParent());

	//set occupation
	pWizard->m_pOccupation = (COccupation*)m_wndOccupation.GetItemData( m_iOccupation );

	//build the investigator's skill list, depending on occupation
    if( pWizard->m_Investigator.SetOccupationSkills( pWizard->m_pOccupation ) == FALSE ) return -1;

	//copy the name out
	m_wndOccupation.GetLBText( m_iOccupation, pWizard->m_Investigator.m_strOccupation );

	//go to next page
	return CPropertyPage::OnWizardNext();
}



















/////////////////////////////////////////////////////////////////////////////
// CNewCharacter_PersonalInterest property page

CNewCharacter_PersonalInterest::CNewCharacter_PersonalInterest() : CPropertyPage(CNewCharacter_PersonalInterest::IDD)
{
	//{{AFX_DATA_INIT(CNewCharacter_PersonalInterest)
	m_strInterestPoints = _T("");
	//}}AFX_DATA_INIT

	m_nPersonalInterestPoints = 0;
}

CNewCharacter_PersonalInterest::~CNewCharacter_PersonalInterest()
{
}

void CNewCharacter_PersonalInterest::DoDataExchange(CDataExchange* pDX)
{
	//finish editing (and set the edit's parent back to normal)
	//before updating everything
	m_wndSkillEdit.EndEditing();

	//set the skill points string
	m_strInterestPoints.Format( "%d", m_nPersonalInterestPoints  );

	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CNewCharacter_PersonalInterest)
	DDX_Control(pDX, IDC_SKILLLIST, m_wndListBox);
	DDX_Control(pDX, IDC_SKILLEDIT, m_wndSkillEdit);
	DDX_Text(pDX, IDC_POINTS, m_strInterestPoints);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CNewCharacter_PersonalInterest, CPropertyPage)
	//{{AFX_MSG_MAP(CNewCharacter_PersonalInterest)
	ON_WM_DRAWITEM()
	ON_WM_MEASUREITEM()
	ON_LBN_DBLCLK(IDC_SKILLLIST, OnDblclkSkilllist)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CNewCharacter_Weapons property page

CNewCharacter_Weapons::CNewCharacter_Weapons() : CPropertyPage(CNewCharacter_Weapons::IDD)
{
	//{{AFX_DATA_INIT(CNewCharacter_Weapons)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	m_pWeaponList = NULL;
}

CNewCharacter_Weapons::~CNewCharacter_Weapons()
{
}

void CNewCharacter_Weapons::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CNewCharacter_Weapons)
	DDX_Control(pDX, IDC_INVESTIGATORWEAPONSLIST, m_wndInvestigatorWeapons);
	DDX_Control(pDX, IDC_AVAILABLEWEAPONSLIST, m_wndAvailableWeapons);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CNewCharacter_Weapons, CPropertyPage)
	//{{AFX_MSG_MAP(CNewCharacter_Weapons)
	ON_BN_CLICKED(IDC_ADDWEAPON, OnAddWeapon)
	ON_LBN_DBLCLK(IDC_AVAILABLEWEAPONSLIST, OnDblclkAvailableWeaponsList)
	ON_LBN_DBLCLK(IDC_INVESTIGATORWEAPONSLIST, OnDblclkInvestigatorWeaponsList)
	ON_BN_CLICKED(IDC_REMOVE, OnRemove)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CNewCharacter_Skills property page

CNewCharacter_Skills::CNewCharacter_Skills() : CPropertyPage(CNewCharacter_Skills::IDD)
{
	m_nSkillPoints = 0;

	//{{AFX_DATA_INIT(CNewCharacter_Skills)
	m_strPoints = _T("");
	//}}AFX_DATA_INIT
}

CNewCharacter_Skills::~CNewCharacter_Skills()
{
}

void CNewCharacter_Skills::DoDataExchange(CDataExchange* pDX)
{
	//finish editing (and set the edit's parent back to normal)
	//before updating everything
	m_wndSkillEdit.EndEditing();

	//set the skill points string
	m_strPoints.Format( "%d", m_nSkillPoints );

	//actually do the data exchange now...
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CNewCharacter_Skills)
	DDX_Control(pDX, IDC_SKILLLIST, m_wndListBox);
	DDX_Control(pDX, IDC_SKILLEDIT, m_wndSkillEdit);
	DDX_Text(pDX, IDC_POINTS, m_strPoints);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CNewCharacter_Skills, CPropertyPage)
	//{{AFX_MSG_MAP(CNewCharacter_Skills)
	ON_WM_DRAWITEM()
	ON_LBN_DBLCLK(IDC_SKILLLIST, OnDblclkSkillList)
	ON_WM_MEASUREITEM()
	ON_BN_CLICKED(IDC_RESET, OnResetSkills)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CNewCharacter_NameEtc property page

CNewCharacter_NameEtc::CNewCharacter_NameEtc() : CPropertyPage(CNewCharacter_NameEtc::IDD)
{
	//{{AFX_DATA_INIT(CNewCharacter_NameEtc)
	m_strBirthplace = _T("");
	m_strColleges = _T("");
	m_strName = _T("");
	m_strSex = _T("");
	m_strMentalDisorders = _T("");
	m_strOccupation = _T("");
	m_strPortrait = _T("");
	//}}AFX_DATA_INIT
}

CNewCharacter_NameEtc::~CNewCharacter_NameEtc()
{
}

void CNewCharacter_NameEtc::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CNewCharacter_NameEtc)
	DDX_Control(pDX, IDC_MENTALDISORDERS, m_wndMentalDisorders);
	DDX_Control(pDX, IDC_BIRTHPLACE, m_wndBirthplace);
	DDX_Control(pDX, IDC_RANDOMNAME, m_wndRandom);
	DDX_CBString(pDX, IDC_BIRTHPLACE, m_strBirthplace);
	DDV_MaxChars(pDX, m_strBirthplace, 100);
	DDX_Text(pDX, IDC_COLLEGES, m_strColleges);
	DDV_MaxChars(pDX, m_strColleges, 100);
	DDX_Text(pDX, IDC_NAME, m_strName);
	DDV_MaxChars(pDX, m_strName, 100);
	DDX_Text(pDX, IDC_SEX, m_strSex);
	DDV_MaxChars(pDX, m_strSex, 1);
	DDX_Text(pDX, IDC_MENTALDISORDERS, m_strMentalDisorders);
	DDV_MaxChars(pDX, m_strMentalDisorders, 200);
	DDX_Text(pDX, IDC_OCCUPATION, m_strOccupation);
	DDV_MaxChars(pDX, m_strOccupation, 100);
	DDX_Text(pDX, IDC_PORTRAIT, m_strPortrait);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CNewCharacter_NameEtc, CPropertyPage)
	//{{AFX_MSG_MAP(CNewCharacter_NameEtc)
	ON_BN_CLICKED(IDC_RANDOMNAME, OnRandomName)
	ON_BN_CLICKED(IDC_BROWSEFORPORTRAIT, OnBrowseForPortrait)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CNewCharacter_Finished property page

CNewCharacter_Finished::CNewCharacter_Finished() : CPropertyPage(CNewCharacter_Finished::IDD)
{
	//{{AFX_DATA_INIT(CNewCharacter_Finished)
	//}}AFX_DATA_INIT
}

CNewCharacter_Finished::~CNewCharacter_Finished()
{
}

void CNewCharacter_Finished::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CNewCharacter_Finished)
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CNewCharacter_Finished, CPropertyPage)
	//{{AFX_MSG_MAP(CNewCharacter_Finished)
		// NOTE: the ClassWizard will add message map macros here
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()






BOOL CNewCharacter_Skills::OnSetActive() 
{
	((CPropertySheet*)GetParent())->SetWizardButtons( PSWIZB_NEXT );
	
    //clear the list box
	m_wndListBox.ResetContent();

	CNewCharacterWizard* pWizard = (CNewCharacterWizard*)GetParent();
    m_nSkillPoints = pWizard->m_nSkillPoints;
	UpdateData(FALSE);

    //add all skills
	InitialiseSkillsList();

	return CPropertyPage::OnSetActive();
}

BOOL CNewCharacter_Finished::OnSetActive() 
{
	((CPropertySheet*)GetParent())->SetWizardButtons( PSWIZB_FINISH );
	return CPropertyPage::OnSetActive();
}

BOOL CNewCharacter_NameEtc::OnSetActive() 
{
	((CPropertySheet*)GetParent())->SetWizardButtons( PSWIZB_BACK|PSWIZB_NEXT );

	m_wndRandom.SendMessage( BM_SETIMAGE, IMAGE_ICON, (LPARAM)LoadImage( AfxGetInstanceHandle(), MAKEINTRESOURCE(IDI_RANDOM), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR ) );
	
	CNewCharacterWizard* pWizard = (CNewCharacterWizard*)GetParent();
	m_strOccupation = pWizard->m_Investigator.m_strOccupation;

    //load in some default options into the birthplace list box
    m_wndBirthplace.ResetContent();
    CString* pstrOptions = GetOptions( "Birthplace" );
	if( pstrOptions != NULL )
	{
		int i = 0;
		while( pstrOptions[i].GetLength() > 0 )
		{
			m_wndBirthplace.AddString( pstrOptions[i] );
			i++;
		}

        delete[] pstrOptions;
	}

    //load in some default options into the phobias list box
    pstrOptions = GetOptions( "Phobias" );
	if( pstrOptions != NULL )
	{
		int i = 0;
		while( pstrOptions[i].GetLength() > 0 )
		{
			m_wndMentalDisorders.AddString( pstrOptions[i] );
			i++;
		}

        delete[] pstrOptions;
	}


	UpdateData(FALSE);

	return CPropertyPage::OnSetActive();
}

BOOL CNewCharacter_PersonalInterest::OnSetActive() 
{
	((CPropertySheet*)GetParent())->SetWizardButtons( PSWIZB_NEXT );

	CNewCharacterWizard* pWizard = (CNewCharacterWizard*)GetParent();

	//add all of the investigator's skills to the list box
	m_nPersonalInterestPoints = pWizard->m_nPersonalInterestPoints;
	UpdateData(FALSE);

	CInvestigator* pInvestigator = &pWizard->m_Investigator;

	//only add everything once
	if( m_wndListBox.GetCount() == 0 )
    {
	    //add current skills
        CInvestigator::InvestigatorSkillList* pCharSkills = pInvestigator->GetList();
	    while( pCharSkills )
	    {
		    if( pCharSkills->pSkill->GetType() != ANYSKILL && ( pCharSkills->pSkill->GetType() != SPECIALISESKILL ) && pCharSkills->pSkill->m_strBase[0] != '*' )
		    {
                int i;

                if( pCharSkills->pSkill->GetType() == SPECIALISESKILLITEM )
                    //file it under it's parent's name
		            i = m_wndListBox.AddString( ((CSpecialiseSkillSkill*)pCharSkills->pSkill)->m_pSkill->m_strName );
                else
                    //list it normally
		            i = m_wndListBox.AddString( pCharSkills->pSkill->m_strName );

                m_wndListBox.SetItemData( i, (DWORD)pCharSkills );
		    }

		    pCharSkills = pCharSkills->next;
	    }
    }

	return CPropertyPage::OnSetActive();
}

BOOL CNewCharacter_Weapons::OnSetActive() 
{
	((CPropertySheet*)GetParent())->SetWizardButtons( PSWIZB_BACK|PSWIZB_NEXT );
	
	CNewCharacterWizard* pWizard = ((CNewCharacterWizard*)GetParent());

	ERA era = pWizard->m_Investigator.m_era;
    m_wndAvailableWeapons.ResetContent();

	int nBaseUnits = GetDialogBaseUnits();
	int nTabStops[] = { 14*nBaseUnits };

	m_wndAvailableWeapons.SetTabStops( sizeof(nTabStops)/sizeof(int), nTabStops );
	m_wndInvestigatorWeapons.SetTabStops( sizeof(nTabStops)/sizeof(int), nTabStops );

	//add the firearms
    CSkillList::SkillList* pSkills = g_Firearms.GetList();
    BOOL fDoneMelee = FALSE;
	while( pSkills )
	{
        //only add this era's weapons
        if( pSkills->pSkill->IsEra( era ) )
		{
            //don't add core melee weapons like grapple and fist/punch
            if( pSkills->pSkill->m_strName[0] != '*' )
            {
			    if( ( pWizard->m_Investigator.FindSkill( pSkills->pSkill->m_strName ) != NULL ) )
			    {
				    int iItem = m_wndInvestigatorWeapons.AddString( pSkills->pSkill->m_strName );
				    m_wndInvestigatorWeapons.SetItemData( iItem, (DWORD)NULL );
			    }
			    else
			    {
				    int iItem = m_wndAvailableWeapons.AddString( pSkills->pSkill->m_strName );
				    m_wndAvailableWeapons.SetItemData( iItem, (DWORD)pSkills->pSkill );
			    }
            }
		}

        //advance list
		pSkills = pSkills->next;

        //after adding firearms, add the melee weapons
        if( pSkills == NULL && fDoneMelee == FALSE )
        {
            pSkills = g_Melee.GetList();
            fDoneMelee = TRUE;
        }
	}

	return CPropertyPage::OnSetActive();
}

void CNewCharacter_Skills::OnDrawItem(int nIDCtl, LPDRAWITEMSTRUCT lpDrawItemStruct) 
{
	if( nIDCtl == IDC_SKILLLIST )
        DrawSkillList( this, nIDCtl, lpDrawItemStruct );		
	else
		CPropertyPage::OnDrawItem(nIDCtl, lpDrawItemStruct);
}

void CNewCharacter_Skills::OnDblclkSkillList() 
{
	int iItem = m_wndListBox.GetCurSel();
    CInvestigator::InvestigatorSkillList* pSkill = ((CInvestigator::InvestigatorSkillList*)m_wndListBox.GetItemData( iItem ));

	CNewCharacterWizard* pWizard = (CNewCharacterWizard*)GetParent();
	if( pSkill->pSkill->GetType() == ANYSKILL )
	{
		CPickSkill PickSkill;
		PickSkill.m_strCustomPrompt = "Please select a skill as a personal specialty";
		PickSkill.m_pInvestigator = &pWizard->m_Investigator;
 		if( PickSkill.DoModal() == IDOK ) 
        {
            //remove the ANYSKILL
            pWizard->m_Investigator.RemoveSkill( pSkill->pSkill );

            //add all skill items
            m_wndListBox.ResetContent();
            InitialiseSkillsList();
        }
	}
	else
	{
		if( pSkill->pSkill->GetType() == SPECIALISESKILLITEM && ((CSpecialiseSkillSkill*)pSkill->pSkill)->m_strName.CompareNoCase( "" ) == 0 )
		{
			CSpecialityNameDlg Speciality;
			Speciality.m_strSkill = ((CSpecialiseSkillSkill*)pSkill->pSkill)->m_pSkill->m_strName;
			if( Speciality.DoModal() == IDOK )
			{
				((CSpecialiseSkillSkill*)pSkill->pSkill)->m_strName = Speciality.m_strSpeciality;
				m_wndListBox.RedrawWindow( NULL, NULL, RDW_INVALIDATE|RDW_UPDATENOW );
			}
		}
		else
			m_wndSkillEdit.BeginEditing( &m_wndListBox, &pWizard->m_Investigator, &m_nSkillPoints );
	}
}


void CNewCharacter_Skills::OnMeasureItem(int nIDCtl, LPMEASUREITEMSTRUCT lpMeasureItemStruct) 
{
	if( nIDCtl == IDC_SKILLLIST )
	{
		lpMeasureItemStruct->itemHeight = 20;
	}
	else
		CPropertyPage::OnMeasureItem(nIDCtl, lpMeasureItemStruct);
}


LRESULT CNewCharacter_Skills::OnWizardNext() 
{
	CNewCharacterWizard* pWizard = (CNewCharacterWizard*)GetParent();

	UpdateData();

	//make sure they leave as happy customers,
	//after spending all their money...
	if( m_nSkillPoints > 0 )
	{
		CString str;
		str.Format( "You still have %d points left unallocated!\nThese are not transferable to other skills and will be lost!\nDo you wish to continue?", m_nSkillPoints );
		
		if( AfxMessageBox( str, MB_YESNO|MB_ICONEXCLAMATION ) == IDNO )
			return -1;
	}

    //zoom through and change the skill's minumum value to whatever it currently is, so
    //that they can't gain extra personal interest points by decreasing an occ. skill
    CInvestigator::InvestigatorSkillList* pList = pWizard->m_Investigator.GetList();
    while( pList )
    {
        pList->nMinimumValue = pList->nCurrent;
        pList = pList->next;
    }


	//store everything again
	pWizard->m_nSkillPoints = m_nSkillPoints;

	return CPropertyPage::OnWizardNext();
}


void CNewCharacter_Skills::OnResetSkills() 
{
    if( AfxMessageBox( "Are you sure you want to reset the skill allocation? (This action cannot be undone)", MB_YESNO ) == IDYES )
    {
	    //refresh the skills list
	    CNewCharacterWizard* pWizard = (CNewCharacterWizard*)GetParent();

        //clear the list box
	    m_wndListBox.ResetContent();

        //reset the investigator's skill list
        if( pWizard->m_Investigator.SetOccupationSkills( pWizard->m_pOccupation ) )
        {
            //add all of the weapons they're carrying
            pWizard->m_Weapons.InitialiseWeaponsList();
        } 
        //else they pressed cancel and the skill list remains unchanged - just add 'em all back to the list box

	    //reset this page
	    InitialiseSkillsList();
		m_nSkillPoints = pWizard->m_nSkillPoints;

	    UpdateData(FALSE);
    }
}

void CNewCharacter_Skills::InitialiseSkillsList()
{
	CNewCharacterWizard* pWizard = (CNewCharacterWizard*)GetParent();

    //add all this chap's skills
    CInvestigator::InvestigatorSkillList* pSkills = pWizard->m_Investigator.GetList();

	while( pSkills )
	{
		int i;

        if( pSkills->fOccupational == TRUE ) //only add occupational skills to the list
        {
            if( pSkills->pSkill->GetType() != SPECIALISESKILL )
            {
                if( pSkills->pSkill->GetType() == SPECIALISESKILLITEM )
                {
                    //file it under it's parent's name
		            i = m_wndListBox.AddString( ((CSpecialiseSkillSkill*)pSkills->pSkill)->m_pSkill->m_strName );
                }
                else
                    //list it normally
		            i = m_wndListBox.AddString( pSkills->pSkill->m_strName );

		        m_wndListBox.SetItemData( i, (DWORD)pSkills );
            }
        }

		pSkills = pSkills->next;
	}
}

void CNewCharacter_PersonalInterest::OnDrawItem(int nIDCtl, LPDRAWITEMSTRUCT lpDrawItemStruct) 
{
	if( nIDCtl == IDC_SKILLLIST )
        DrawSkillList( this, nIDCtl, lpDrawItemStruct );		
	else
		CPropertyPage::OnDrawItem(nIDCtl, lpDrawItemStruct);
}

void CNewCharacter_PersonalInterest::OnMeasureItem(int nIDCtl, LPMEASUREITEMSTRUCT lpMeasureItemStruct) 
{
	if( nIDCtl == IDC_SKILLLIST )
	{
		lpMeasureItemStruct->itemHeight = 20;
	}
	else
		CPropertyPage::OnMeasureItem(nIDCtl, lpMeasureItemStruct);
}

void CNewCharacter_PersonalInterest::OnDblclkSkilllist() 
{
	int iItem = m_wndListBox.GetCurSel();
    CSkill* pSkill = ((CInvestigator::InvestigatorSkillList*)m_wndListBox.GetItemData( iItem ))->pSkill;

	CNewCharacterWizard* pWizard = (CNewCharacterWizard*)GetParent();
	if( pSkill->GetType() == ANYSKILL )
	{
		TRACE0( "Unexpected ANYSKILL!" );
		ASSERT(FALSE);
	}
	else
	{
		if( pSkill->GetType() == SPECIALISESKILLITEM && ((CSpecialiseSkillSkill*)pSkill)->m_strName.CompareNoCase( "" ) == 0 )
		{
			CSpecialityNameDlg Speciality;
			Speciality.m_strSkill = ((CSpecialiseSkillSkill*)pSkill)->m_pSkill->m_strName;
			if( Speciality.DoModal() == IDOK )
			{
				((CSpecialiseSkillSkill*)pSkill)->m_strName = Speciality.m_strSpeciality;
				m_wndListBox.RedrawWindow( NULL, NULL, RDW_INVALIDATE|RDW_UPDATENOW );
			}
		}
		else
			m_wndSkillEdit.BeginEditing( &m_wndListBox, &pWizard->m_Investigator, &m_nPersonalInterestPoints );
	}
}

LRESULT CNewCharacter_PersonalInterest::OnWizardNext() 
{
	CNewCharacterWizard* pWizard = (CNewCharacterWizard*)GetParent();
	pWizard->m_nPersonalInterestPoints = m_nPersonalInterestPoints;
	

	if( m_nPersonalInterestPoints > 0 )
	{
		CString str;
		str.Format( "You still have %d points left unallocated - these will be lost!\nDo you wish to continue?", m_nPersonalInterestPoints );
		
		if( AfxMessageBox( str, MB_YESNO|MB_ICONEXCLAMATION ) == IDNO )
			return -1;
	}


	return CPropertyPage::OnWizardNext();
}

void CNewCharacter_NameEtc::OnRandomName() 
{
	UpdateData();
    GetRandomName( m_strName, m_strSex );
	UpdateData( FALSE );
}

LRESULT CNewCharacter_NameEtc::OnWizardNext() 
{
	CNewCharacterWizard* pWizard = (CNewCharacterWizard*)GetParent();

	UpdateData();

	pWizard->m_Investigator.m_strName = m_strName;
	pWizard->m_Investigator.m_strOccupation = m_strOccupation;
	pWizard->m_Investigator.m_strBirthplace = m_strBirthplace;
	pWizard->m_Investigator.m_strColleges = m_strColleges;
	pWizard->m_Investigator.m_strMentalDisorders = m_strMentalDisorders;
	pWizard->m_Investigator.m_strSex = m_strSex;
    pWizard->m_Investigator.m_strPortrait = m_strPortrait;
	
	return CPropertyPage::OnWizardNext();
}

void CNewCharacter_Weapons::OnDblclkAvailableWeaponsList() 
{
	OnAddWeapon();
}

void CNewCharacter_Weapons::OnDblclkInvestigatorWeaponsList() 
{
	OnRemove();
}

void CNewCharacter_Weapons::OnAddWeapon() 
{
    //count the number of melee and firearm weapons selected so far...
    int iMax = m_wndInvestigatorWeapons.GetCount();
    int nMelee = 0, nFirearm = 0;

    for( int index = 0; index < iMax; index++ )
    {
        CSkill* pSkill = (CSkill*)m_wndInvestigatorWeapons.GetItemData( index );
        if( pSkill->GetType() == MELEE ) nMelee++;
        else
            if( pSkill->GetType() == FIREARM ) nFirearm++;
                else
                    TRACE0("Unknown type in firearm list!!\n");
    }

    //add the skill to the list box
	int i = m_wndAvailableWeapons.GetCurSel();

	if( i != LB_ERR )
	{
        //get the selected skill
		CString str;
		CSkill* pSkill = (CSkill*)m_wndAvailableWeapons.GetItemData( i );

        //let them know if there is too many firearms or melee weapons
        if( pSkill->GetType() == MELEE && nMelee == 4 )
            if( IDNO == AfxMessageBox( "Note: Only the first four hand-to-hand weapons\nare displayed on the investigator sheet!\nDo you want to add this one anyway?", MB_ICONEXCLAMATION|MB_YESNO ) )
                return;
            
        if( pSkill->GetType() == FIREARM && nFirearm == 9 )
            if( IDNO == AfxMessageBox( "Note: Only the first nine firearms\nare displayed on the investigator sheet!\nDo you want to add this one anyway?", MB_ICONEXCLAMATION|MB_YESNO ) )
                return;

        //get it's caption and remove it from the available weapons
		m_wndAvailableWeapons.GetText( i, str );
		m_wndAvailableWeapons.DeleteString( i );

        //add it to the investigator's weapons and attach the skill to it again
		i = m_wndInvestigatorWeapons.AddString( str );
		m_wndInvestigatorWeapons.SetItemData( i, (DWORD)pSkill );
	}
}

void CNewCharacter_Weapons::OnRemove() 
{
	int i = m_wndInvestigatorWeapons.GetCurSel();

	if( i != LB_ERR )
	{
		CString str;
		DWORD dwData = m_wndInvestigatorWeapons.GetItemData( i );
        if( dwData == NULL )
        {
            AfxMessageBox( "Sorry - You can't delete occupational weapons", MB_ICONEXCLAMATION );
        }
        else
        {
		    m_wndInvestigatorWeapons.GetText( i, str );
		    m_wndInvestigatorWeapons.DeleteString( i );

		    i = m_wndAvailableWeapons.AddString( str );
		    m_wndAvailableWeapons.SetItemData( i, dwData );
        }
	}
}

void CNewCharacter_Weapons::InitialiseWeaponsList()
{
	CNewCharacterWizard* pWizard = (CNewCharacterWizard*)GetParent();
	int max = m_wndInvestigatorWeapons.GetCount();

	for( int i = 0; i < max; i++ )
	{
        CSkill* pSkill = (CSkill*)m_wndInvestigatorWeapons.GetItemData(i);

		//add skill to investigator
		if( pSkill ) 
        {
            CInvestigator::InvestigatorSkillList* pList = pWizard->m_Investigator.AddSkill( pSkill, FALSE );
            pList->nMinimumValue = pList->nCurrent = pWizard->m_Investigator.GetSkillNumber( pSkill->m_strBase );
        }
	}
}

LRESULT CNewCharacter_Weapons::OnWizardNext() 
{
    InitialiseWeaponsList();
	return CPropertyPage::OnWizardNext();
}

void CNewCharacter_NameEtc::OnBrowseForPortrait() 
{
    UpdateData();

    OPENFILENAME ofn;
    char szFilename[MAX_PATH+1];
    static CString strCurrentDir;
    LPCTSTR lpstrFileMasks = "Bitmap Files\0*.bmp\0All Files (*.*)\0*.*\0\0";

    if( strCurrentDir == "" ) //first call to this function (static just initialised?)
    {
        //set default directory to 'personal folders' from shell info.
        LPITEMIDLIST itemId;
        char szPersonalFolder[MAX_PATH+1];

        SHGetSpecialFolderLocation( m_hWnd, CSIDL_PERSONAL, &itemId );
        SHGetPathFromIDList( itemId, szPersonalFolder );

        strCurrentDir = szPersonalFolder;
    }

    //get the current file name as default
    strcpy( szFilename, LPCSTR(m_strPortrait) );
    
    //browse for a new one
    ofn.lStructSize = sizeof(OPENFILENAME);
    ofn.hwndOwner = m_hWnd;
    ofn.hInstance = NULL;
    ofn.lpstrFilter = lpstrFileMasks;
    ofn.lpstrCustomFilter = NULL;
    ofn.nFilterIndex = 1;
    ofn.lpstrFile = szFilename;
    ofn.nMaxFile = MAX_PATH;
    ofn.lpstrFileTitle = NULL;
    ofn.nMaxFileTitle = 0;
    ofn.lpstrInitialDir = LPCSTR(strCurrentDir);
    ofn.lpstrTitle = "Select Portrait";
    ofn.Flags = OFN_EXPLORER|OFN_HIDEREADONLY|OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST;
    ofn.lpstrDefExt = "BMP";
    if( GetOpenFileName( &ofn ) )
    {
        //change it
        m_strPortrait = ofn.lpstrFile;
        UpdateData(FALSE);
    }
}
