// Skill.h: interface for the CSkill class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SKILL_H__B534ECD6_A26B_11D1_89C0_00001C192944__INCLUDED_)
#define AFX_SKILL_H__B534ECD6_A26B_11D1_89C0_00001C192944__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

enum ERA { ERA_1890, ERA_1920, ERA_1990 };
enum SkillType { NO_TYPE, SKILL, FIREARM, MELEE, ANYSKILL, SPECIALISESKILL, SPECIALISESKILLITEM };

//COC file header values
#define SHEET_FILE_HEADER10 "COC10"
#define SHEET_FILE_HEADER20 "COC20"

class CSkill {

public:
    CSkill( CString strName, CString strBase, BOOL f1890, BOOL f1920, BOOL f1990 );
    CSkill();
    virtual ~CSkill();

    virtual CSkill* GetCopy();

    BOOL IsEra( ERA era );

    SkillType GetType();

    int CompareName( CSkill* pOtherSkill );
    int CompareName( CString pString );

    BOOL IsCalculatedBase();

    void AddRef();
    void RemoveRef();

    CString m_strName;
    CString m_strBase;
    BOOL m_f1890, m_f1920, m_f1990;

protected:
    SkillType m_Type;
    int m_nRef;
};

class CSpecialiseSkill : public CSkill {

public:
    CSpecialiseSkill( int nSkills, CString strName, CString strBase, BOOL f1890, BOOL f1920, BOOL f1990, BOOL fInitialise = TRUE );
    virtual ~CSpecialiseSkill();
    virtual CSkill* GetCopy();

    class CSpecialiseSkillSkill** m_pSkillSpecialities;
    int m_nSkills; //total number of skills in this subset
};



class CSpecialiseSkillSkill : public CSkill {

public:
    CSpecialiseSkillSkill();
    CSpecialiseSkill* m_pSkill;
    virtual CSkill* GetCopy();

};

class CFirearmSkill : public CSkill {

public:
    CFirearmSkill( CString strName, CString strBase, BOOL f1890, BOOL f1920, BOOL f1990, CString strDmg, CString strRange, CString strShots, CString strBullets, CString strHP, CString strMAL );
    virtual ~CFirearmSkill();
    virtual CSkill* GetCopy();

	CString m_strDmg;
	CString m_strRange;
	CString m_strShots;
	CString m_strBullets;
	CString m_strHP;
	CString m_strMAL;
};

class CMeleeSkill : public CSkill {

public:
    CMeleeSkill(CString strName, CString strBase, BOOL f1890, BOOL f1920, BOOL f1990, CString strDmg, CString strAttacks, CString strHP );
    virtual ~CMeleeSkill();
    virtual CSkill* GetCopy();

	CString m_strDmg;
	CString m_strAttacks;
	CString m_strHP;
};



class CSkillList {

public:
    CSkillList();
    virtual ~CSkillList();

    struct SkillList {
        CSkill* pSkill;
        SkillList* next;
    };

    BOOL AddSkill( CSkill* pSkill );
    CSkill* FindSkill( CString strName );

    SkillList* GetList() { return m_pSkillList; };

    void ResetList();

    int GetCount();


    //assignment operator
    const CSkillList& operator=(const CSkillList& src );


protected:
    SkillList* m_pSkillList;
};


class COccupation {

public:
    COccupation();
    virtual ~COccupation();

    CString m_strName;
    BOOL m_f1890, m_f1920, m_f1990;
    BOOL IsEra( ERA era );

    BOOL AddSkill( CSkill* pSkill ) { return m_SkillList.AddSkill( pSkill ); };
    BOOL AddChoice( CSkill* pSkill) { return m_ChoiceList.AddSkill( pSkill ); };
    void SetChoiceCount( int n ) { m_nChoice = n; };
    int GetChoiceCount() { return m_nChoice; };
    
    CSkillList& GetSkillList() { return m_SkillList; };
    CSkillList& GetChoiceList() { return m_ChoiceList; };

protected:
    CSkillList m_SkillList;

    int m_nChoice;
    CSkillList m_ChoiceList;
};


class COccupationList {

public:
    COccupationList();
    virtual ~COccupationList();

    struct OccupationList {
        COccupation* pOccupation;
        OccupationList* next;
    };

    BOOL AddOccupation( COccupation* pOccupation );
    COccupation* FindOccupation( CString strName );

    void ResetList();

    int GetCount();

    OccupationList* GetList() { return m_pOccupationList; };


protected:

    OccupationList* m_pOccupationList;

};


class CInvestigator {

public:
    CInvestigator( CInvestigator& src );
    CInvestigator();
    virtual ~CInvestigator();

    //storage stuff
    void Serialize(CArchive& ar);
    int ReadSkill( CArchive& ar, BOOL fOccupational, int nCurrent, int nMinimumValue );
    void WriteSkill( CArchive& ar, CSkill* pSkill );

    struct InvestigatorSkillList {
        CSkill* pSkill;
        int nCurrent;
        int nMinimumValue;
        BOOL fOccupational;
        InvestigatorSkillList* next;
    };

    int FindStatValue( const char* pszStatName );
    int FindSkillValue( const char* pszSkillName );
    InvestigatorSkillList* AddSkill( CSkill* pSkill, BOOL fOccupational );
    void RemoveSkill( CSkill* pSkill );
    InvestigatorSkillList* FindSkill( CString strName );
    void EvaluateSkills();
    void DeleteSkills();
    BOOL SetOccupationSkills( COccupation* pOccupation );
    int GetSkillNumber( const char* pszValue );
    void SortSkillList();
    BOOL IsBase( int nPercent, CSkill* pSkill );

    void ClearWeapons();

    InvestigatorSkillList* GetList() { return m_pSkillList; };
    
	CString m_strName;
	CString m_strOccupation;
	CString m_strBirthplace, m_strColleges, m_strMentalDisorders, m_strSex;
	int m_nAge;
	ERA m_era;

    CString m_strPortrait;
	
	int m_nAPP;
	int m_nCON;
	int m_nDEX;
	int m_nEDU;
	int m_nINT;
	int m_nPOW;
	int m_nSIZ;
	int m_nSTR;

	CString m_strPlayerName;

    //NEW FOR VERSION 1.1:
    int m_nSAN;
    int m_nHitPoints;
    int m_nMagic;
    CString m_strDG_Affiliation;
    CString m_strDG_CodeName;
    CString m_strDG_CurrentDate;
    CString m_strDG_Insanities;

    //assignment operator
    const CInvestigator& operator=(const CInvestigator& src );

protected:
    InvestigatorSkillList* m_pSkillList;

private:
    InvestigatorSkillList* Sort_RemoveLargest( InvestigatorSkillList* start );
	int CompareSkills( CSkill* pSkill1, CSkill* pSkill2 );
	InvestigatorSkillList* AddSkillCopy( CSkill* pSkill, BOOL fOccupational );

    void CopyInvestigator( const CInvestigator& src );
};


extern CSkillList g_CoreSkills;
extern CSkillList g_Firearms;
extern CSkillList g_Melee;

extern COccupationList g_Occupations;

CSkill* FindSkill( char* pszSkill );

BOOL LoadDataTables();

CString GetSkillCode( const char* pszValue );
CString GetDBString( int nStr, int nSiz );
CString Abbreviate( const char* pszText );
void GetRandomName( CString& strName, CString& strSex );

CString* GetOptions( const char* pszSection, const char* pszFile = "OPTIONS.LST" );

#endif // !defined(AFX_SKILL_H__B534ECD6_A26B_11D1_89C0_00001C192944__INCLUDED_)
