// UserTextEdit.cpp : implementation file
//

#include "stdafx.h"
#include "cthulhu.h"
#include "CthulhuDoc.h"
#include "CthulhuView.h"
#include "UserTextEdit.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CUserTextEdit

CUserTextEdit::CUserTextEdit()
{
    m_fEditing = FALSE;
    m_EditingType = EDITING_NONE;
    m_crColour = 0x000000;

    m_hWndSpin = NULL;
}

CUserTextEdit::~CUserTextEdit()
{
    m_Font.DeleteObject();
}


BEGIN_MESSAGE_MAP(CUserTextEdit, CEdit)
	//{{AFX_MSG_MAP(CUserTextEdit)
	ON_WM_KEYDOWN()
	ON_WM_GETDLGCODE()
	ON_WM_KILLFOCUS()
	ON_WM_CTLCOLOR_REFLECT()
	ON_CONTROL_REFLECT(EN_CHANGE, OnChange)
	ON_WM_CHAR()
	ON_WM_KEYUP()
	ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CUserTextEdit message handlers

BOOL CUserTextEdit::Editing()
{
    return m_fEditing;
}

void CUserTextEdit::EndEditing(BOOL fSaveChanges)
{
    if( m_fEditing )
    {
        m_fEditing = FALSE;
        if( fSaveChanges && m_fTextChanged )
        {
            //store an undo
            ((CCthulhuView*)GetParent())->GetDocument()->StoreUndo( m_strDescription );

            //extract the new value
            CString str;
            GetWindowText( str );
            if( str.GetLength() > 0 )
            {
                //decode new value
                switch( m_EditingType )
                {
                    case EDITING_NONE:
                        //nothing to do...
                        break;

                    case EDITING_TEXT:
                        *m_pstrText = str;
                        break;

                    case EDITING_NUMBER:
                    {
                        int n = atoi( str );
                        if( n < m_nMinValue ) n = m_nMinValue;
                        *m_pnValue = n;
                        break;
                    }
                }

                //update the document
                ((CCthulhuView*)GetParent())->GetDocument()->SetModifiedFlag();
                ((CCthulhuView*)GetParent())->GetDocument()->m_fNeedRedrawRecalculation = TRUE;
                ((CCthulhuView*)GetParent())->GetDocument()->UpdateAllViews( NULL );
            }
            else MessageBeep(0xFFFFFFFF);
        }

        //hide the editing window
        m_EditingType = EDITING_NONE;
        ShowWindow( SW_HIDE );
        MoveWindow( -1, -1, -1, -1, FALSE );
    }

    if( m_hWndSpin )
    {
        ::DestroyWindow( m_hWndSpin );
        m_hWndSpin = NULL;
    }
}

void CUserTextEdit::BeginEditing(CString * pstr, CRect rc, CString strDescription, int nMaxLength )
{
    EndEditing( TRUE );
    m_strDescription = strDescription;

    m_EditingType = EDITING_TEXT;
    rc.bottom += 6;
    SetMargins( 0, 0 );
    MoveWindow( &rc, FALSE );

    ModifyStyle( ES_NUMBER, 0 );

    SetWindowText( *pstr );
    LimitText( nMaxLength );
    m_pstrText = pstr;
    m_fEditing = TRUE;

    ShowWindow( SW_SHOW );
    SetFocus();

    EmptyUndoBuffer();
    SetSel( 0, -1 );
    m_fTextChanged = FALSE;
}

void CUserTextEdit::BeginEditing(int * pn, CRect rc, CString strDescription, int nMinValue )
{
    EndEditing( TRUE );
    m_strDescription = strDescription;

    m_EditingType = EDITING_NUMBER;
    
    rc.bottom += 6;
    rc.right += 16; //allow for spin control too
    SetMargins( 0, 0 );

    MoveWindow( &rc, FALSE );

    if( *pn >= 0 )
    {
        CString str;
        str.Format( "%d", *pn );
        SetWindowText( str );
    }
    else SetWindowText( "" );
    LimitText( 3 );

    ModifyStyle( 0, ES_NUMBER );

    m_nMinValue = nMinValue;
    m_pnValue = pn;
    m_fEditing = TRUE;

    ShowWindow( SW_SHOW );
    SetFocus();

    EmptyUndoBuffer();
    SetSel( 0, -1 );

    m_hWndSpin = ::CreateUpDownControl( WS_BORDER|WS_VISIBLE|WS_CHILD|UDS_ARROWKEYS|UDS_ALIGNRIGHT|UDS_SETBUDDYINT|UDS_NOTHOUSANDS, 0, 0, 30, 30, GetParent()->m_hWnd, 0xFFFF, AfxGetInstanceHandle(), m_hWnd, 999, nMinValue, *pn );
    m_fTextChanged = FALSE;
}

void CUserTextEdit::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	switch( nChar )
	{
		case VK_RETURN:
			break;

		case VK_ESCAPE:
			break;

        case '*':
            MessageBeep(0xFFFFFFFF);
            break;

        default:
	        CEdit::OnKeyDown(nChar, nRepCnt, nFlags);
            break;
	}

}
void CUserTextEdit::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	switch( nChar )
	{
		case VK_RETURN:
			EndEditing( TRUE );
			break;

		case VK_ESCAPE:
			EndEditing( FALSE );
			break;

        case '*':
            MessageBeep(0xFFFFFFFF);
            break;

        default:
        	CEdit::OnKeyUp(nChar, nRepCnt, nFlags);
    }
}

void CUserTextEdit::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	switch( nChar )
	{
		case VK_RETURN:
			break;

		case VK_ESCAPE:
			break;

        case '*':
            MessageBeep(0xFFFFFFFF);
            break;

        default:
	        CEdit::OnChar(nChar, nRepCnt, nFlags);
    }
}

UINT CUserTextEdit::OnGetDlgCode() 
{
	if( m_fEditing )
		return DLGC_WANTALLKEYS|DLGC_WANTARROWS|DLGC_WANTCHARS;
	else
		return CEdit::OnGetDlgCode();
}

void CUserTextEdit::OnKillFocus(CWnd* pNewWnd) 
{
    EndEditing( TRUE );
	CEdit::OnKillFocus(pNewWnd);
}

void CUserTextEdit::CreateFont(CString strFace, COLORREF crColour)
{
    m_Font.DeleteObject();
    m_Font.CreateFont( -11, 0, 0, 0, FW_NORMAL, 0, FALSE, 0, 0, 0, 0, 0, 0, strFace );
    SetFont( &m_Font );

    m_crColour = crColour;
}

HBRUSH CUserTextEdit::CtlColor(CDC* pDC, UINT nCtlColor) 
{
    pDC->SetTextColor( m_crColour );
	return (HBRUSH)GetStockObject( WHITE_BRUSH );
}


void CUserTextEdit::OnChange() 
{
    m_fTextChanged = TRUE;
}




void CUserTextEdit::OnEditUndo() 
{
    SendMessage( WM_UNDO );	
}

BOOL CUserTextEdit::PreCreateWindow(CREATESTRUCT& cs) 
{
    cs.dwExStyle |= WS_EX_CLIENTEDGE;
	return CEdit::PreCreateWindow(cs);
}
